import base64
import json
import re
import urllib.parse

from .common import InfoExtractor, Request
from ..utils import (
    determine_ext,
    int_or_none,
    js_to_json,
    parse_duration,
    parse_iso8601,
    url_or_none,
)
from ..utils.traversal import traverse_obj


class RTPIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?rtp\.pt/play/(?:[^/#?]+/)?p(?P<program_id>\d+)/(?P<id>e\d+)'
    _TESTS = [{
        'url': 'http://www.rtp.pt/play/p405/e174042/paixoes-cruzadas',
        'md5': 'e736ce0c665e459ddb818546220b4ef8',
        'info_dict': {
            'id': 'e174042',
            'ext': 'mp3',
            'title': 'Paixões Cruzadas',
            'description': 'md5:af979e58ba0ab73f78435fc943fdb070',
            'thumbnail': r're:^https?://.*\.jpg',
            'series': 'Paixões Cruzadas',
            'duration': 2950.0,
            'modified_timestamp': 1553693464,
            'modified_date': '20190327',
            'timestamp': 1417219200,
            'upload_date': '20141129',
        },
    }, {
        'url': 'https://www.rtp.pt/play/zigzag/p13166/e757904/25-curiosidades-25-de-abril',
        'md5': '5b4859940e3adef61247a77dfb76046a',
        'info_dict': {
            'id': 'e757904',
            'ext': 'mp4',
            'title': 'Estudar ou não estudar',
            'description': 'md5:3bfd7eb8bebfd5711a08df69c9c14c35',
            'thumbnail': r're:^https?://.*\.jpg',
            'timestamp': 1711958401,
            'duration': 146.0,
            'upload_date': '20240401',
            'modified_timestamp': 1712242991,
            'series': '25 Curiosidades, 25 de Abril',
            'episode_number': 2,
            'episode': 'Estudar ou não estudar',
            'modified_date': '20240404',
        },
    }, {
        # Episode not accessible through API
        'url': 'https://www.rtp.pt/play/estudoemcasa/p7776/e500050/portugues-1-ano',
        'md5': '57660c0b46db9f22118c52cbd65975e4',
        'info_dict': {
            'id': 'e500050',
            'ext': 'mp4',
            'title': 'Português - 1.º ano',
            'duration': 1669.0,
            'description': 'md5:be68925c81269f8c6886589f25fe83ea',
            'upload_date': '20201020',
            'timestamp': 1603180799,
            'thumbnail': 'https://cdn-images.rtp.pt/EPG/imagens/39482_59449_64850.png?v=3&w=860',
        },
    }]

    _USER_AGENT = 'rtpplay/2.0.66 (pt.rtp.rtpplay; build:2066; iOS 15.8.3) Alamofire/5.9.1'
    _AUTH_TOKEN = None

    def _fetch_auth_token(self):
        if self._AUTH_TOKEN:
            return self._AUTH_TOKEN
        self._AUTH_TOKEN = traverse_obj(self._download_json(Request(
            'https://rtpplayapi.rtp.pt/play/api/2/token-manager',
            headers={
                'Accept': '*/*',
                'rtp-play-auth': 'RTPPLAY_MOBILE_IOS',
                'rtp-play-auth-hash': 'fac9c328b2f27e26e03d7f8942d66c05b3e59371e16c2a079f5c83cc801bd3ee',
                'rtp-play-auth-timestamp': '2145973229682',
                'User-Agent': self._USER_AGENT,
            }, extensions={'keep_header_casing': True}), None,
            note='Fetching guest auth token', errnote='Could not fetch guest auth token',
            fatal=False), ('token', 'token', {str}))
        return self._AUTH_TOKEN

    @staticmethod
    def _cleanup_media_url(url):
        if urllib.parse.urlparse(url).netloc == 'streaming-ondemand.rtp.pt':
            return None
        return url.replace('/drm-fps/', '/hls/').replace('/drm-dash/', '/dash/')

    def _extract_formats(self, media_urls, episode_id):
        formats = []
        subtitles = {}
        for media_url in set(traverse_obj(media_urls, (..., {url_or_none}, {self._cleanup_media_url}))):
            ext = determine_ext(media_url)
            if ext == 'm3u8':
                fmts, subs = self._extract_m3u8_formats_and_subtitles(
                    media_url, episode_id, m3u8_id='hls', fatal=False)
                formats.extend(fmts)
                self._merge_subtitles(subs, target=subtitles)
            elif ext == 'mpd':
                fmts, subs = self._extract_mpd_formats_and_subtitles(
                    media_url, episode_id, mpd_id='dash', fatal=False)
                formats.extend(fmts)
                self._merge_subtitles(subs, target=subtitles)
            else:
                formats.append({
                    'url': media_url,
                    'format_id': 'http',
                })
        return formats, subtitles

    def _extract_from_api(self, program_id, episode_id):
        auth_token = self._fetch_auth_token()
        if not auth_token:
            return
        episode_data = traverse_obj(self._download_json(
            f'https://www.rtp.pt/play/api/1/get-episode/{program_id}/{episode_id[1:]}', episode_id,
            query={'include_assets': 'true', 'include_webparams': 'true'},
            headers={
                'Accept': '*/*',
                'Authorization': f'Bearer {auth_token}',
                'User-Agent': self._USER_AGENT,
            }, fatal=False), 'result', {dict})
        if not episode_data:
            return
        asset_urls = traverse_obj(episode_data, ('assets', 0, 'asset_url', {dict}))
        media_urls = traverse_obj(asset_urls, (
            ((('hls', 'dash'), 'stream_url'), ('multibitrate', ('url_hls', 'url_dash'))),))
        formats, subtitles = self._extract_formats(media_urls, episode_id)

        for sub_data in traverse_obj(asset_urls, ('subtitles', 'vtt_list', lambda _, v: url_or_none(v['file']))):
            subtitles.setdefault(sub_data.get('code') or 'pt', []).append({
                'url': sub_data['file'],
                'name': sub_data.get('language'),
            })

        return {
            'id': episode_id,
            'formats': formats,
            'subtitles': subtitles,
            'thumbnail': traverse_obj(episode_data, ('assets', 0, 'asset_thumbnail', {url_or_none})),
            **traverse_obj(episode_data, ('episode', {
                'title': (('episode_title', 'program_title'), {str}, filter, any),
                'alt_title': ('episode_subtitle', {str}, filter),
                'description': (('episode_description', 'episode_summary'), {str}, filter, any),
                'timestamp': ('episode_air_date', {parse_iso8601(delimiter=' ')}),
                'modified_timestamp': ('episode_lastchanged', {parse_iso8601(delimiter=' ')}),
                'duration': ('episode_duration_complete', {parse_duration}),
                'episode': ('episode_title', {str}, filter),
                'episode_number': ('episode_number', {int_or_none}),
                'season': ('program_season', {str}, filter),
                'series': ('program_title', {str}, filter),
            })),
        }

    _RX_OBFUSCATION = re.compile(r'''(?xs)
        atob\s*\(\s*decodeURIComponent\s*\(\s*
            (\[[0-9A-Za-z%,'"]*\])
        \s*\.\s*join\(\s*(?:""|'')\s*\)\s*\)\s*\)
    ''')

    def __unobfuscate(self, data):
        return self._RX_OBFUSCATION.sub(
            lambda m: json.dumps(
                base64.b64decode(urllib.parse.unquote(
                    ''.join(json.loads(m.group(1))),
                )).decode('iso-8859-1')),
            data)

    def _extract_from_html(self, url, episode_id):
        webpage = self._download_webpage(url, episode_id)

        formats = []
        subtitles = {}
        media_urls = traverse_obj(re.findall(r'(?:var\s+f\s*=|RTPPlayer\({[^}]+file:)\s*({[^}]+}|"[^"]+")', webpage), (
            -1, (({self.__unobfuscate}, {js_to_json}, {json.loads}, {dict.values}, ...), {json.loads})))
        formats, subtitles = self._extract_formats(media_urls, episode_id)

        return {
            'id': episode_id,
            'formats': formats,
            'subtitles': subtitles,
            'description': self._html_search_meta(['og:description', 'twitter:description'], webpage, default=None),
            'thumbnail': self._html_search_meta(['og:image', 'twitter:image'], webpage, default=None),
            **self._search_json_ld(webpage, episode_id, default={}),
            'title': self._html_search_meta(['og:title', 'twitter:title'], webpage, default=None),
        }

    def _real_extract(self, url):
        program_id, episode_id = self._match_valid_url(url).group('program_id', 'id')
        return self._extract_from_api(program_id, episode_id) or self._extract_from_html(url, episode_id)



from .common import InfoExtractor
from .youtube import YoutubeIE, YoutubeTabIE


class BeatBumpVideoIE(InfoExtractor):
    _VALID_URL = r'https?://beatbump\.(?:ml|io)/listen\?id=(?P<id>[\w-]+)'
    _TESTS = [{
        'url': 'https://beatbump.ml/listen?id=MgNrAu2pzNs',
        'md5': '5ff3fff41d3935b9810a9731e485fe66',
        'info_dict': {
            'id': 'MgNrAu2pzNs',
            'ext': 'mp4',
            'artist': 'Stephen',
            'thumbnail': 'https://i.ytimg.com/vi_webp/MgNrAu2pzNs/maxresdefault.webp',
            'channel_url': 'https://www.youtube.com/channel/UC-pWHpBjdGG69N9mM2auIAA',
            'upload_date': '20190312',
            'categories': ['Music'],
            'playable_in_embed': True,
            'duration': 169,
            'like_count': int,
            'alt_title': 'Voyeur Girl',
            'view_count': int,
            'track': 'Voyeur Girl',
            'uploader': 'Stephen',
            'title': 'Voyeur Girl',
            'channel_follower_count': int,
            'age_limit': 0,
            'availability': 'public',
            'live_status': 'not_live',
            'album': 'it\'s too much love to know my dear',
            'channel': 'Stephen',
            'comment_count': int,
            'description': 'md5:7ae382a65843d6df2685993e90a8628f',
            'tags': 'count:11',
            'creator': 'Stephen',
            'channel_id': 'UC-pWHpBjdGG69N9mM2auIAA',
            'channel_is_verified': True,
            'heatmap': 'count:100',
        },
    }, {
        'url': 'https://beatbump.io/listen?id=LDGZAprNGWo',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        id_ = self._match_id(url)
        return self.url_result(f'https://music.youtube.com/watch?v={id_}', YoutubeIE, id_)


class BeatBumpPlaylistIE(InfoExtractor):
    _VALID_URL = r'https?://beatbump\.(?:ml|io)/(?:release\?id=|artist/|playlist/)(?P<id>[\w-]+)'
    _TESTS = [{
        'url': 'https://beatbump.ml/release?id=MPREb_gTAcphH99wE',
        'playlist_count': 50,
        'info_dict': {
            'id': 'OLAK5uy_l1m0thk3g31NmIIz_vMIbWtyv7eZixlH0',
            'availability': 'unlisted',
            'view_count': int,
            'title': 'Album - Royalty Free Music Library V2 (50 Songs)',
            'description': '',
            'tags': [],
            'modified_date': '20231110',
        },
        'expected_warnings': ['YouTube Music is not directly supported'],
    }, {
        'url': 'https://beatbump.ml/artist/UC_aEa8K-EOJ3D6gOs7HcyNg',
        'playlist_mincount': 1,
        'params': {'flatplaylist': True},
        'info_dict': {
            'id': 'UC_aEa8K-EOJ3D6gOs7HcyNg',
            'uploader_url': 'https://www.youtube.com/@NoCopyrightSounds',
            'channel_url': 'https://www.youtube.com/channel/UC_aEa8K-EOJ3D6gOs7HcyNg',
            'uploader_id': '@NoCopyrightSounds',
            'channel_follower_count': int,
            'title': 'NoCopyrightSounds',
            'uploader': 'NoCopyrightSounds',
            'description': 'md5:cd4fd53d81d363d05eee6c1b478b491a',
            'channel': 'NoCopyrightSounds',
            'tags': 'count:65',
            'channel_id': 'UC_aEa8K-EOJ3D6gOs7HcyNg',
            'channel_is_verified': True,
        },
        'expected_warnings': ['YouTube Music is not directly supported'],
    }, {
        'url': 'https://beatbump.ml/playlist/VLPLRBp0Fe2GpgmgoscNFLxNyBVSFVdYmFkq',
        'playlist_mincount': 1,
        'params': {'flatplaylist': True},
        'info_dict': {
            'id': 'PLRBp0Fe2GpgmgoscNFLxNyBVSFVdYmFkq',
            'uploader_url': 'https://www.youtube.com/@NoCopyrightSounds',
            'description': 'Providing you with copyright free / safe music for gaming, live streaming, studying and more!',
            'view_count': int,
            'channel_url': 'https://www.youtube.com/channel/UC_aEa8K-EOJ3D6gOs7HcyNg',
            'uploader_id': '@NoCopyrightSounds',
            'title': 'NCS : All Releases 💿',
            'uploader': 'NoCopyrightSounds',
            'availability': 'public',
            'channel': 'NoCopyrightSounds',
            'tags': [],
            'modified_date': '20231112',
            'channel_id': 'UC_aEa8K-EOJ3D6gOs7HcyNg',
        },
        'expected_warnings': ['YouTube Music is not directly supported'],
    }, {
        'url': 'https://beatbump.io/playlist/VLPLFCHGavqRG-q_2ZhmgU2XB2--ZY6irT1c',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        id_ = self._match_id(url)
        return self.url_result(f'https://music.youtube.com/browse/{id_}', YoutubeTabIE, id_)



from .common import InfoExtractor
from ..utils import clean_html, int_or_none, traverse_obj

_API_URL = 'https://dak1vd5vmi7x6.cloudfront.net/api/v1/publicrole/{}/{}?id={}'


class ManotoTVIE(InfoExtractor):
    IE_DESC = 'Manoto TV (Episode)'
    _VALID_URL = r'https?://(?:www\.)?manototv\.com/episode/(?P<id>[0-9]+)'
    _TESTS = [{
        'url': 'https://www.manototv.com/episode/8475',
        'info_dict': {
            'id': '8475',
            'series': 'خانه های رویایی با برادران اسکات',
            'season_number': 7,
            'episode_number': 25,
            'episode_id': 'My Dream Home S7: Carol & John',
            'duration': 3600,
            'categories': ['سرگرمی'],
            'title': 'کارول و جان',
            'description': 'md5:d0fff1f8ba5c6775d312a00165d1a97e',
            'thumbnail': r're:^https?://.*\.(jpeg|png|jpg)$',
            'ext': 'mp4',
        },
        'params': {
            'skip_download': 'm3u8',
        },
    }, {
        'url': 'https://www.manototv.com/episode/12576',
        'info_dict': {
            'id': '12576',
            'series': 'فیلم های ایرانی',
            'episode_id': 'Seh Mah Taatili',
            'duration': 5400,
            'view_count': int,
            'categories': ['سرگرمی'],
            'title': 'سه ماه تعطیلی',
            'description': 'سه ماه تعطیلی فیلمی به کارگردانی و نویسندگی شاپور قریب ساختهٔ سال ۱۳۵۶ است.',
            'thumbnail': r're:^https?://.*\.(jpeg|png|jpg)$',
            'ext': 'mp4',
        },
        'params': {
            'skip_download': 'm3u8',
        },
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        episode_json = self._download_json(_API_URL.format('showmodule', 'episodedetails', video_id), video_id)
        details = episode_json.get('details', {})
        formats = self._extract_m3u8_formats(details.get('videoM3u8Url'), video_id, 'mp4')
        return {
            'id': video_id,
            'series': details.get('showTitle'),
            'season_number': int_or_none(details.get('analyticsSeasonNumber')),
            'episode_number': int_or_none(details.get('episodeNumber')),
            'episode_id': details.get('analyticsEpisodeTitle'),
            'duration': int_or_none(details.get('durationInMinutes'), invscale=60),
            'view_count': details.get('viewCount'),
            'categories': [details.get('videoCategory')],
            'title': details.get('episodeTitle'),
            'description': clean_html(details.get('episodeDescription')),
            'thumbnail': details.get('episodelandscapeImgIxUrl'),
            'formats': formats,
        }


class ManotoTVShowIE(InfoExtractor):
    IE_DESC = 'Manoto TV (Show)'
    _VALID_URL = r'https?://(?:www\.)?manototv\.com/show/(?P<id>[0-9]+)'
    _TESTS = [{
        'url': 'https://www.manototv.com/show/2526',
        'playlist_mincount': 68,
        'info_dict': {
            'id': '2526',
            'title': 'فیلم های ایرانی',
            'description': 'مجموعه ای از فیلم های سینمای کلاسیک ایران',
        },
    }]

    def _real_extract(self, url):
        show_id = self._match_id(url)
        show_json = self._download_json(_API_URL.format('showmodule', 'details', show_id), show_id)
        show_details = show_json.get('details', {})
        title = show_details.get('showTitle')
        description = show_details.get('showSynopsis')

        series_json = self._download_json(_API_URL.format('showmodule', 'serieslist', show_id), show_id)
        playlist_id = str(traverse_obj(series_json, ('details', 'list', 0, 'id')))

        playlist_json = self._download_json(_API_URL.format('showmodule', 'episodelist', playlist_id), playlist_id)
        playlist = traverse_obj(playlist_json, ('details', 'list')) or []

        entries = [
            self.url_result(
                'https://www.manototv.com/episode/{}'.format(item['slideID']), ie=ManotoTVIE.ie_key(), video_id=item['slideID'])
            for item in playlist]
        return self.playlist_result(entries, show_id, title, description)


class ManotoTVLiveIE(InfoExtractor):
    IE_DESC = 'Manoto TV (Live)'
    _VALID_URL = r'https?://(?:www\.)?manototv\.com/live/'
    _TEST = {
        'url': 'https://www.manototv.com/live/',
        'info_dict': {
            'id': 'live',
            'title': 'Manoto TV Live',
            'ext': 'mp4',
            'is_live': True,
        },
        'params': {
            'skip_download': 'm3u8',
        },
    }

    def _real_extract(self, url):
        video_id = 'live'
        json = self._download_json(_API_URL.format('livemodule', 'details', ''), video_id)
        details = json.get('details', {})
        video_url = details.get('liveUrl')
        formats = self._extract_m3u8_formats(video_url, video_id, 'mp4', live=True)
        return {
            'id': video_id,
            'title': 'Manoto TV Live',
            'is_live': True,
            'formats': formats,
        }



import re

from .common import InfoExtractor
from ..utils import (
    clean_html,
    get_element_by_class,
    parse_qs,
    remove_start,
    unescapeHTML,
    urljoin,
)


class WikimediaIE(InfoExtractor):
    IE_NAME = 'wikimedia.org'
    _VALID_URL = r'https?://commons\.wikimedia\.org/wiki/File:(?P<id>[^/#?]+)\.\w+'
    _TESTS = [{
        'url': 'https://commons.wikimedia.org/wiki/File:Die_Temperaturkurve_der_Erde_(ZDF,_Terra_X)_720p_HD_50FPS.webm',
        'info_dict': {
            'url': 're:https?://upload.wikimedia.org/wikipedia',
            'ext': 'webm',
            'id': 'Die_Temperaturkurve_der_Erde_(ZDF,_Terra_X)_720p_HD_50FPS',
            'title': 'Die Temperaturkurve der Erde (ZDF, Terra X) 720p HD 50FPS.webm - Wikimedia Commons',
            'description': 'md5:7cd84f76e7081f1be033d0b155b4a460',
            'license': 'Creative Commons Attribution 4.0 International',
            'uploader': 'ZDF/Terra X/Gruppe 5/Luise Wagner, Jonas Sichert, Andreas Hougardy',
            'subtitles': 'count:4',
        },
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)

        subtitles = {}
        for sub in set(re.findall(r'\bsrc\s*=\s*["\'](/w/api[^"]+)["\']', webpage)):
            sub = urljoin('https://commons.wikimedia.org', unescapeHTML(sub))
            qs = parse_qs(sub)
            lang = qs.get('lang', [None])[-1]
            sub_ext = qs.get('trackformat', [None])[-1]
            if lang and sub_ext:
                subtitles.setdefault(lang, []).append({'ext': sub_ext, 'url': sub})

        return {
            'id': video_id,
            'url': self._html_search_regex(r'<source\s[^>]*\bsrc="([^"]+)"', webpage, 'video URL'),
            'description': clean_html(get_element_by_class('description', webpage)),
            'title': remove_start(self._og_search_title(webpage), 'File:'),
            'license': self._html_search_regex(
                r'licensed under(?: the)? (.+?) license',
                get_element_by_class('licensetpl', webpage), 'license', default=None),
            'uploader': self._html_search_regex(
                r'>\s*Author\s*</td>\s*<td\b[^>]*>\s*([^<]+)\s*</td>', webpage, 'video author', default=None),
            'subtitles': subtitles,
        }



from .common import InfoExtractor


class VolejTVIE(InfoExtractor):
    _VALID_URL = r'https?://volej\.tv/video/(?P<id>\d+)'
    _TESTS = [{
        'url': 'https://volej.tv/video/725742/',
        'info_dict': {
            'id': '725742',
            'ext': 'mp4',
            'description': 'Zápas VK Královo Pole vs VK Prostějov 10.12.2022 v 19:00 na Volej.TV',
            'thumbnail': 'https://volej.tv/images/og/16/17186/og.png',
            'title': 'VK Královo Pole vs VK Prostějov',
        },
    }, {
        'url': 'https://volej.tv/video/725605/',
        'info_dict': {
            'id': '725605',
            'ext': 'mp4',
            'thumbnail': 'https://volej.tv/images/og/15/17185/og.png',
            'title': 'VK Lvi Praha vs VK Euro Sitex Příbram',
            'description': 'Zápas VK Lvi Praha vs VK Euro Sitex Příbram 11.12.2022 v 19:00 na Volej.TV',
        },
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)
        json_data = self._search_json(
            r'<\s*!\[CDATA[^=]+=', webpage, 'CDATA', video_id)
        formats, subtitle = self._extract_m3u8_formats_and_subtitles(
            json_data['urls']['hls'], video_id)
        return {
            'id': video_id,
            'title': self._html_search_meta(['og:title', 'twitter:title'], webpage),
            'thumbnail': self._html_search_meta(['og:image', 'twitter:image'], webpage),
            'description': self._html_search_meta(['description', 'og:description', 'twitter:description'], webpage),
            'formats': formats,
            'subtitles': subtitle,
        }



from .common import InfoExtractor
from .youtube import YoutubeIE
from ..utils import (
    determine_ext,
    int_or_none,
    mimetype2ext,
    parse_iso8601,
    traverse_obj,
)


class AirTVIE(InfoExtractor):
    _VALID_URL = r'https?://www\.air\.tv/watch\?v=(?P<id>\w+)'
    _TESTS = [{
        # without youtube_id
        'url': 'https://www.air.tv/watch?v=W87jcWleSn2hXZN47zJZsQ',
        'info_dict': {
            'id': 'W87jcWleSn2hXZN47zJZsQ',
            'ext': 'mp4',
            'release_date': '20221003',
            'release_timestamp': 1664792603,
            'channel_id': 'vgfManQlRQKgoFQ8i8peFQ',
            'title': 'md5:c12d49ed367c3dadaa67659aff43494c',
            'upload_date': '20221003',
            'duration': 151,
            'view_count': int,
            'thumbnail': 'https://cdn-sp-gcs.air.tv/videos/W/8/W87jcWleSn2hXZN47zJZsQ/b13fc56464f47d9d62a36d110b9b5a72-4096x2160_9.jpg',
            'timestamp': 1664792603,
        },
    }, {
        # with youtube_id
        'url': 'https://www.air.tv/watch?v=sv57EC8tRXG6h8dNXFUU1Q',
        'info_dict': {
            'id': '2ZTqmpee-bQ',
            'ext': 'mp4',
            'comment_count': int,
            'tags': 'count:11',
            'channel_follower_count': int,
            'like_count': int,
            'uploader': 'Newsflare',
            'thumbnail': 'https://i.ytimg.com/vi_webp/2ZTqmpee-bQ/maxresdefault.webp',
            'availability': 'public',
            'title': 'Geese Chase Alligator Across Golf Course',
            'uploader_id': 'NewsflareBreaking',
            'channel_url': 'https://www.youtube.com/channel/UCzSSoloGEz10HALUAbYhngQ',
            'description': 'md5:99b21d9cea59330149efbd9706e208f5',
            'age_limit': 0,
            'channel_id': 'UCzSSoloGEz10HALUAbYhngQ',
            'uploader_url': 'http://www.youtube.com/user/NewsflareBreaking',
            'view_count': int,
            'categories': ['News & Politics'],
            'live_status': 'not_live',
            'playable_in_embed': True,
            'channel': 'Newsflare',
            'duration': 37,
            'upload_date': '20180511',
        },
    }]

    def _get_formats_and_subtitle(self, json_data, video_id):
        formats, subtitles = [], {}
        for source in traverse_obj(json_data, 'sources', 'sources_desktop', ...):
            ext = determine_ext(source.get('src'), mimetype2ext(source.get('type')))
            if ext == 'm3u8':
                fmts, subs = self._extract_m3u8_formats_and_subtitles(source.get('src'), video_id)
                formats.extend(fmts)
                self._merge_subtitles(subs, target=subtitles)
            else:
                formats.append({'url': source.get('src'), 'ext': ext})
        return formats, subtitles

    def _real_extract(self, url):
        display_id = self._match_id(url)
        webpage = self._download_webpage(url, display_id)

        nextjs_json = self._search_nextjs_data(webpage, display_id)['props']['pageProps']['initialState']['videos'][display_id]
        if nextjs_json.get('youtube_id'):
            return self.url_result(
                f'https://www.youtube.com/watch?v={nextjs_json.get("youtube_id")}', YoutubeIE)

        formats, subtitles = self._get_formats_and_subtitle(nextjs_json, display_id)
        return {
            'id': display_id,
            'title': nextjs_json.get('title') or self._html_search_meta('og:title', webpage),
            'formats': formats,
            'subtitles': subtitles,
            'description': nextjs_json.get('description') or None,
            'duration': int_or_none(nextjs_json.get('duration')),
            'thumbnails': [
                {'url': thumbnail}
                for thumbnail in traverse_obj(nextjs_json, ('default_thumbnails', ...))],
            'channel_id': traverse_obj(nextjs_json, 'channel', 'channel_slug'),
            'timestamp': parse_iso8601(nextjs_json.get('created')),
            'release_timestamp': parse_iso8601(nextjs_json.get('published')),
            'view_count': int_or_none(nextjs_json.get('views')),
        }
